@@ -8,6 +8,8 @@
 #
 ######################################################################
 
+from typing import Optional
+
 from .exception import MissingAccountData
 from .upload_url_pool import UrlPoolAccountInfo
 
@@ -53,29 +55,25 @@ class InMemoryAccountInfo(UrlPoolAccountInfo):
         self._auth_token = None
         self._buckets = {}
         self._download_url = None
-        self._minimum_part_size = None
+        self._recommended_part_size = None
+        self._absolute_minimum_part_size = None
         self._realm = None
+        self._s3_api_url = None
 
     def _set_auth_data(
-        self,
-        account_id,
-        auth_token,
-        api_url,
-        download_url,
-        minimum_part_size,
-        application_key,
-        realm,
-        allowed,
-        application_key_id,
+        self, account_id, auth_token, api_url, download_url, recommended_part_size,
+        absolute_minimum_part_size, application_key, realm, s3_api_url, allowed, application_key_id
     ):
         self._account_id = account_id
         self._application_key_id = application_key_id
         self._auth_token = auth_token
         self._api_url = api_url
         self._download_url = download_url
-        self._minimum_part_size = minimum_part_size
+        self._absolute_minimum_part_size = absolute_minimum_part_size
+        self._recommended_part_size = recommended_part_size
         self._application_key = application_key
         self._realm = realm
+        self._s3_api_url = s3_api_url
         self._allowed = allowed
 
     def refresh_entire_bucket_name_cache(self, name_id_iterable):
@@ -84,6 +82,12 @@ class InMemoryAccountInfo(UrlPoolAccountInfo):
     def get_bucket_id_or_none_from_bucket_name(self, bucket_name):
         return self._buckets.get(bucket_name)
 
+    def get_bucket_name_or_none_from_bucket_id(self, bucket_id: str) -> Optional[str]:
+        for name, cached_id_ in self._buckets.items():
+            if cached_id_ == bucket_id:
+                return name
+        return None
+
     def save_bucket(self, bucket):
         self._buckets[bucket.name] = bucket.id_
 
@@ -116,8 +120,12 @@ class InMemoryAccountInfo(UrlPoolAccountInfo):
         return self._download_url
 
     @_raise_missing_if_result_is_none
-    def get_minimum_part_size(self):
-        return self._minimum_part_size
+    def get_recommended_part_size(self):
+        return self._recommended_part_size
+
+    @_raise_missing_if_result_is_none
+    def get_absolute_minimum_part_size(self):
+        return self._absolute_minimum_part_size
 
     @_raise_missing_if_result_is_none
     def get_realm(self):
@@ -126,3 +134,7 @@ class InMemoryAccountInfo(UrlPoolAccountInfo):
     @_raise_missing_if_result_is_none
     def get_allowed(self):
         return self._allowed
+
+    @_raise_missing_if_result_is_none
+    def get_s3_api_url(self):
+        return self._s3_api_url
