@@ -8,6 +8,7 @@
 #
 ######################################################################
 
+from random import random
 import io
 import json
 import logging
@@ -15,14 +16,17 @@ import socket
 
 import arrow
 import requests
+from requests.adapters import HTTPAdapter
 import time
 
-from typing import Any, Dict
+from typing import Any, Dict, Optional
 
 from .exception import (
     B2Error, B2RequestTimeoutDuringUpload, BadDateFormat, BrokenPipe, B2ConnectionError,
     B2RequestTimeout, ClockSkew, ConnectionReset, interpret_b2_error, UnknownError, UnknownHost
 )
+from .api_config import B2HttpApiConfig, DEFAULT_HTTP_API_CONFIG
+from .requests import NotDecompressingResponse
 from .version import USER_AGENT
 
 logger = logging.getLogger(__name__)
@@ -42,102 +46,7 @@ def _print_exception(e, indent=''):
             _print_exception(a, indent + '        ')
 
 
-def _translate_errors(fcn, post_params=None):
-    """
-    Call the given function, turning any exception raised into the right
-    kind of B2Error.
-
-    :param dict post_params: request parameters
-    """
-    try:
-        response = fcn()
-        if response.status_code not in [200, 206]:
-            # Decode the error object returned by the service
-            error = json.loads(response.content.decode('utf-8')) if response.content else {}
-            raise interpret_b2_error(
-                int(error.get('status', response.status_code)),
-                error.get('code'),
-                error.get('message'),
-                response.headers,
-                post_params,
-            )
-        return response
-
-    except B2Error:
-        raise  # pass through exceptions from just above
-
-    except requests.ConnectionError as e0:
-        e1 = e0.args[0]
-        if isinstance(e1, requests.packages.urllib3.exceptions.MaxRetryError):
-            msg = e1.args[0]
-            if 'nodename nor servname provided, or not known' in msg:
-                # Unknown host, or DNS failing.  In the context of calling
-                # B2, this means that something is down between here and
-                # Backblaze, so we treat it like 503 Service Unavailable.
-                raise UnknownHost()
-        elif isinstance(e1, requests.packages.urllib3.exceptions.ProtocolError):
-            e2 = e1.args[1]
-            if isinstance(e2, socket.error):
-                if len(e2.args) >= 2 and e2.args[1] == 'Broken pipe':
-                    # Broken pipes are usually caused by the service rejecting
-                    # an upload request for cause, so we use a 400 Bad Request
-                    # code.
-                    raise BrokenPipe()
-        raise B2ConnectionError(str(e0))
-
-    except requests.Timeout as e:
-        raise B2RequestTimeout(str(e))
-
-    except Exception as e:
-        text = repr(e)
-
-        # This is a special case to handle when urllib3 doesn't translate
-        # ECONNRESET into something that requests can turn into something
-        # we understand.  The SysCallError is from the optional library
-        # pyOpenSsl, which we don't require, so we can't import it and
-        # catch it explicitly.
-        #
-        # The text from one such error looks like this: SysCallError(104, 'ECONNRESET')
-        if text.startswith('SysCallError'):
-            if 'ECONNRESET' in text:
-                raise ConnectionReset()
-
-        logger.exception('_translate_errors has intercepted an unexpected exception')
-        raise UnknownError(text)
-
-
-def _translate_and_retry(fcn, try_count, post_params=None):
-    """
-    Try calling fcn try_count times, retrying only if
-    the exception is a retryable B2Error.
-
-    :param int try_count: a number of retries
-    :param dict post_params: request parameters
-    """
-    # For all but the last try, catch the exception.
-    wait_time = 1.0
-    for _ in range(try_count - 1):
-        try:
-            return _translate_errors(fcn, post_params)
-        except B2Error as e:
-            if not e.should_retry_http():
-                raise
-            logger.debug(str(e), exc_info=True)
-            if e.retry_after_seconds is not None:
-                sleep_duration = e.retry_after_seconds
-                sleep_reason = 'server asked us to'
-            else:
-                sleep_duration = wait_time
-                sleep_reason = 'that is what the default exponential backoff is'
-            logger.info('Pausing thread for %i seconds because %s', sleep_duration, sleep_reason)
-            time.sleep(sleep_duration)
-            wait_time *= 1.5
-
-    # If the last try gets an exception, it will be raised.
-    return _translate_errors(fcn, post_params)
-
-
-class ResponseContextManager(object):
+class ResponseContextManager:
     """
     A context manager that closes a requests.Response when done.
     """
@@ -152,7 +61,7 @@ class ResponseContextManager(object):
         self.response.close()
 
 
-class HttpCallback(object):
+class HttpCallback:
     """
     A callback object that does nothing.  Overrides pre_request
     and/or post_request as desired.
@@ -222,7 +131,7 @@ class ClockSkewHook(HttpCallback):
             raise ClockSkew(skew_seconds)
 
 
-class B2Http(object):
+class B2Http:
     """
     A wrapper for the requests module.  Provides the operations
     needed to access B2, and handles retrying when the returned
@@ -245,25 +154,30 @@ class B2Http(object):
        except B2Error as e:
            ...
 
+    Please note that the timeout/retry system, including class-level variables,
+    is not a part of the interface and is subject to change.
     """
 
-    # timeout for HTTP GET/POST requests
-    TIMEOUT = 900  # 15 minutes as server-side copy can take time
+    TIMEOUT = 128
+    TIMEOUT_FOR_COPY = 1200  # 20 minutes as server-side copy can take time
+    TIMEOUT_FOR_UPLOAD = 128
+    TRY_COUNT_DATA = 20
+    TRY_COUNT_DOWNLOAD = 20
+    TRY_COUNT_HEAD = 5
+    TRY_COUNT_OTHER = 5
 
-    def __init__(self, requests_module=None, install_clock_skew_hook=True, user_agent_append=None):
+    def __init__(self, api_config: B2HttpApiConfig = DEFAULT_HTTP_API_CONFIG):
         """
         Initialize with a reference to the requests module, which makes
         it easy to mock for testing.
-
-        :param requests_module: a reference to requests module
-        :param bool install_clock_skew_hook: if True, install a clock skew hook
-        :param str user_agent_append: if provided, the string will be appended to the User-Agent
         """
-        requests_to_use = requests_module or requests
-        self.user_agent = self._get_user_agent(user_agent_append)
-        self.session = requests_to_use.Session()
+        self.user_agent = self._get_user_agent(api_config.user_agent_append)
+        self.session = api_config.http_session_factory()
+        if not api_config.decode_content:
+            self.session.adapters.clear()
+            self.session.mount('', NotDecompressingHTTPAdapter())
         self.callbacks = []
-        if install_clock_skew_hook:
+        if api_config.install_clock_skew_hook:
             self.add_callback(ClockSkewHook())
 
     def add_callback(self, callback):
@@ -275,7 +189,15 @@ class B2Http(object):
         """
         self.callbacks.append(callback)
 
-    def post_content_return_json(self, url, headers, data, try_count=5, post_params=None):
+    def post_content_return_json(
+        self,
+        url,
+        headers,
+        data,
+        try_count: int = TRY_COUNT_DATA,
+        post_params=None,
+        _timeout: Optional[int] = None,
+    ):
         """
         Use like this:
 
@@ -301,13 +223,16 @@ class B2Http(object):
             data.seek(0)
             self._run_pre_request_hooks('POST', url, request_headers)
             response = self.session.post(
-                url, headers=request_headers, data=data, timeout=self.TIMEOUT
+                url,
+                headers=request_headers,
+                data=data,
+                timeout=_timeout or self.TIMEOUT_FOR_UPLOAD,
             )
             self._run_post_request_hooks('POST', url, request_headers, response)
             return response
 
         try:
-            response = _translate_and_retry(do_post, try_count, post_params)
+            response = self._translate_and_retry(do_post, try_count, post_params)
         except B2RequestTimeout:
             # this forces a token refresh, which is necessary if request is still alive
             # on the server but has terminated for some reason on the client. See #79
@@ -322,7 +247,7 @@ class B2Http(object):
         finally:
             response.close()
 
-    def post_json_return_json(self, url, headers, params, try_count=5):
+    def post_json_return_json(self, url, headers, params, try_count: int = TRY_COUNT_OTHER):
         """
         Use like this:
 
@@ -340,10 +265,24 @@ class B2Http(object):
         :return: the decoded JSON document
         :rtype: dict
         """
-        data = io.BytesIO(json.dumps(params).encode())
-        return self.post_content_return_json(url, headers, data, try_count, params)
 
-    def get_content(self, url, headers, try_count=5):
+        # This is not just b2_copy_file or b2_copy_part, but it would not
+        # be good to find out by analyzing the url.
+        # In the future a more generic system between raw_api and b2http
+        # to indicate the timeouts should be designed.
+        timeout = self.TIMEOUT_FOR_COPY
+
+        data = io.BytesIO(json.dumps(params).encode())
+        return self.post_content_return_json(
+            url,
+            headers,
+            data,
+            try_count,
+            params,
+            _timeout=timeout,
+        )
+
+    def get_content(self, url, headers, try_count: int = TRY_COUNT_DOWNLOAD):
         """
         Fetches content from a URL.
 
@@ -378,10 +317,15 @@ class B2Http(object):
             self._run_post_request_hooks('GET', url, request_headers, response)
             return response
 
-        response = _translate_and_retry(do_get, try_count, None)
+        response = self._translate_and_retry(do_get, try_count, None)
         return ResponseContextManager(response)
 
-    def head_content(self, url: str, headers: Dict[str, Any], try_count: int = 5) -> Dict[str, Any]:
+    def head_content(
+        self,
+        url: str,
+        headers: Dict[str, Any],
+        try_count: int = TRY_COUNT_HEAD,
+    ) -> Dict[str, Any]:
         """
         Does a HEAD instead of a GET for the URL.
         The response's content is limited to the headers.
@@ -416,7 +360,7 @@ class B2Http(object):
             self._run_post_request_hooks('HEAD', url, request_headers, response)
             return response
 
-        return _translate_and_retry(do_head, try_count, None)
+        return self._translate_and_retry(do_head, try_count, None)
 
     @classmethod
     def _get_user_agent(cls, user_agent_append):
@@ -432,6 +376,128 @@ class B2Http(object):
         for callback in self.callbacks:
             callback.post_request(method, url, headers, response)
 
+    @classmethod
+    def _translate_errors(cls, fcn, post_params=None):
+        """
+        Call the given function, turning any exception raised into the right
+        kind of B2Error.
+
+        :param dict post_params: request parameters
+        """
+        try:
+            response = fcn()
+            if response.status_code not in [200, 206]:
+                # Decode the error object returned by the service
+                error = json.loads(response.content.decode('utf-8')) if response.content else {}
+                extra_error_keys = error.keys() - ('code', 'status', 'message')
+                if extra_error_keys:
+                    logger.debug(
+                        'received error has extra (unsupported) keys: %s', extra_error_keys
+                    )
+                raise interpret_b2_error(
+                    int(error.get('status', response.status_code)),
+                    error.get('code'),
+                    error.get('message'),
+                    response.headers,
+                    post_params,
+                )
+            return response
+
+        except B2Error:
+            raise  # pass through exceptions from just above
+
+        except requests.ConnectionError as e0:
+            e1 = e0.args[0]
+            if isinstance(e1, requests.packages.urllib3.exceptions.MaxRetryError):
+                msg = e1.args[0]
+                if 'nodename nor servname provided, or not known' in msg:
+                    # Unknown host, or DNS failing.  In the context of calling
+                    # B2, this means that something is down between here and
+                    # Backblaze, so we treat it like 503 Service Unavailable.
+                    raise UnknownHost()
+            elif isinstance(e1, requests.packages.urllib3.exceptions.ProtocolError):
+                e2 = e1.args[1]
+                if isinstance(e2, socket.error):
+                    if len(e2.args) >= 2 and e2.args[1] == 'Broken pipe':
+                        # Broken pipes are usually caused by the service rejecting
+                        # an upload request for cause, so we use a 400 Bad Request
+                        # code.
+                        raise BrokenPipe()
+            raise B2ConnectionError(str(e0))
+
+        except requests.Timeout as e:
+            raise B2RequestTimeout(str(e))
+
+        except Exception as e:
+            text = repr(e)
+
+            # This is a special case to handle when urllib3 doesn't translate
+            # ECONNRESET into something that requests can turn into something
+            # we understand.  The SysCallError is from the optional library
+            # pyOpenSsl, which we don't require, so we can't import it and
+            # catch it explicitly.
+            #
+            # The text from one such error looks like this: SysCallError(104, 'ECONNRESET')
+            if text.startswith('SysCallError'):
+                if 'ECONNRESET' in text:
+                    raise ConnectionReset()
+
+            logger.exception('_translate_errors has intercepted an unexpected exception')
+            raise UnknownError(text)
+
+    @classmethod
+    def _translate_and_retry(cls, fcn, try_count, post_params=None):
+        """
+        Try calling fcn try_count times, retrying only if
+        the exception is a retryable B2Error.
+
+        :param int try_count: a number of retries
+        :param dict post_params: request parameters
+        """
+        # For all but the last try, catch the exception.
+        wait_time = 1.0
+        max_wait_time = 64
+        for _ in range(try_count - 1):
+            try:
+                return cls._translate_errors(fcn, post_params)
+            except B2Error as e:
+                if not e.should_retry_http():
+                    raise
+                logger.debug(str(e), exc_info=True)
+                if e.retry_after_seconds is not None:
+                    sleep_duration = e.retry_after_seconds
+                    sleep_reason = 'server asked us to'
+                else:
+                    sleep_duration = wait_time
+                    sleep_reason = 'that is what the default exponential backoff is'
+
+                logger.info(
+                    'Pausing thread for %i seconds because %s',
+                    sleep_duration,
+                    sleep_reason,
+                )
+                time.sleep(sleep_duration)
+
+                # Set up wait time for the next iteration
+                wait_time *= 1.5
+                if wait_time > max_wait_time:
+                    # avoid clients synchronizing and causing a wave
+                    # of requests when connectivity is restored
+                    wait_time = max_wait_time + random()
+
+        # If the last try gets an exception, it will be raised.
+        return cls._translate_errors(fcn, post_params)
+
+
+class NotDecompressingHTTPAdapter(HTTPAdapter):
+    """
+    HTTP adapter that uses :class:`b2sdk.requests.NotDecompressingResponse` instead of the default
+    :code:`requests.Response` class.
+    """
+
+    def build_response(self, req, resp):
+        return NotDecompressingResponse.from_builtin_response(super().build_response(req, resp))
+
 
 def test_http():
     """
