@@ -10,14 +10,22 @@
 
 from functools import partial
 from enum import Enum, unique
-from typing import Any, Dict
+from typing import Any, Dict, Optional
+import logging
 
+from b2sdk.account_info.abstract import AbstractAccountInfo
 from b2sdk.account_info.sqlite_account_info import SqliteAccountInfo
 from b2sdk.account_info.exception import MissingAccountData
 from b2sdk.b2http import B2Http
-from b2sdk.cache import AuthInfoCache, DummyCache
+from b2sdk.cache import AbstractCache, AuthInfoCache, DummyCache
+from b2sdk.encryption.setting import EncryptionSetting
+from b2sdk.replication.setting import ReplicationConfiguration
 from b2sdk.exception import (InvalidAuthToken, Unauthorized)
-from b2sdk.raw_api import ALL_CAPABILITIES, B2RawApi
+from b2sdk.file_lock import BucketRetentionSetting, FileRetentionSetting, LegalHold
+from b2sdk.raw_api import ALL_CAPABILITIES, REALM_URLS
+from b2sdk.api_config import B2HttpApiConfig, DEFAULT_HTTP_API_CONFIG
+
+logger = logging.getLogger(__name__)
 
 
 @unique
@@ -28,21 +36,28 @@ class TokenType(Enum):
     UPLOAD_SMALL = 'upload_small'
 
 
-class B2Session(object):
+class B2Session:
     """
         A facade that supplies the correct api_url and account_auth_token
         to methods of underlying raw_api and reauthorizes if necessary.
     """
+    SQLITE_ACCOUNT_INFO_CLASS = staticmethod(SqliteAccountInfo)
+    B2HTTP_CLASS = staticmethod(B2Http)
 
-    def __init__(self, account_info=None, cache=None, raw_api=None):
+    def __init__(
+        self,
+        account_info: Optional[AbstractAccountInfo] = None,
+        cache: Optional[AbstractCache] = None,
+        api_config: B2HttpApiConfig = DEFAULT_HTTP_API_CONFIG
+    ):
         """
         Initialize Session using given account info.
 
-        :param account_info: an instance of :class:`~b2sdk.v1.UrlPoolAccountInfo`,
+        :param account_info: an instance of :class:`~b2sdk.v2.UrlPoolAccountInfo`,
                       or any custom class derived from
-                      :class:`~b2sdk.v1.AbstractAccountInfo`
+                      :class:`~b2sdk.v2.AbstractAccountInfo`
                       To learn more about Account Info objects, see here
-                      :class:`~b2sdk.v1.SqliteAccountInfo`
+                      :class:`~b2sdk.v2.SqliteAccountInfo`
 
         :param cache: an instance of the one of the following classes:
                       :class:`~b2sdk.cache.DummyCache`, :class:`~b2sdk.cache.InMemoryCache`,
@@ -51,17 +66,12 @@ class B2Session(object):
                       It is used by B2Api to cache the mapping between bucket name and bucket ids.
                       default is :class:`~b2sdk.cache.DummyCache`
 
-        :param raw_api: an instance of one of the following classes:
-                        :class:`~b2sdk.raw_api.B2RawApi`, :class:`~b2sdk.raw_simulator.RawSimulator`,
-                        or any custom class derived from :class:`~b2sdk.raw_api.AbstractRawApi`
-                        It makes network-less unit testing simple by using :class:`~b2sdk.raw_simulator.RawSimulator`,
-                        in tests and :class:`~b2sdk.raw_api.B2RawApi` in production.
-                        default is :class:`~b2sdk.raw_api.B2RawApi`
+        :param api_config
         """
 
-        self.raw_api = raw_api or B2RawApi(B2Http())
+        self.raw_api = api_config.raw_api_class(self.B2HTTP_CLASS(api_config))
         if account_info is None:
-            account_info = SqliteAccountInfo()
+            account_info = self.SQLITE_ACCOUNT_INFO_CLASS()
             if cache is None:
                 cache = AuthInfoCache(account_info)
         if cache is None:
@@ -99,31 +109,29 @@ class B2Session(object):
         :param str application_key_id: :term:`application key ID`
         :param str application_key: user's :term:`application key`
         """
-        # Clean up any previous account info if it was for a different account.
-        try:
-            old_account_id = self.account_info.get_account_id()
-            old_realm = self.account_info.get_realm()
-            if application_key_id != old_account_id or realm != old_realm:
-                self.cache.clear()
-        except MissingAccountData:
-            self.cache.clear()
-
         # Authorize
-        realm_url = self.account_info.REALM_URLS.get(realm, realm)
+        realm_url = REALM_URLS.get(realm, realm)
         response = self.raw_api.authorize_account(realm_url, application_key_id, application_key)
+        account_id = response['accountId']
         allowed = response['allowed']
 
+        # Clear the cache if new account has been used
+        if not self.account_info.is_same_account(account_id, realm):
+            self.cache.clear()
+
         # Store the auth data
         self.account_info.set_auth_data(
-            response['accountId'],
-            response['authorizationToken'],
-            response['apiUrl'],
-            response['downloadUrl'],
-            response['recommendedPartSize'],
-            application_key,
-            realm,
-            allowed,
-            application_key_id,
+            account_id=account_id,
+            auth_token=response['authorizationToken'],
+            api_url=response['apiUrl'],
+            download_url=response['downloadUrl'],
+            absolute_minimum_part_size=response['absoluteMinimumPartSize'],
+            recommended_part_size=response['recommendedPartSize'],
+            application_key=application_key,
+            realm=realm,
+            s3_api_url=response['s3ApiUrl'],
+            allowed=allowed,
+            application_key_id=application_key_id
         )
 
     def cancel_large_file(self, file_id):
@@ -136,7 +144,10 @@ class B2Session(object):
         bucket_type,
         bucket_info=None,
         cors_rules=None,
-        lifecycle_rules=None
+        lifecycle_rules=None,
+        default_server_side_encryption=None,
+        is_file_lock_enabled: Optional[bool] = None,
+        replication: Optional[ReplicationConfiguration] = None,
     ):
         return self._wrap_default_token(
             self.raw_api.create_bucket,
@@ -146,6 +157,9 @@ class B2Session(object):
             bucket_info=bucket_info,
             cors_rules=cors_rules,
             lifecycle_rules=lifecycle_rules,
+            default_server_side_encryption=default_server_side_encryption,
+            is_file_lock_enabled=is_file_lock_enabled,
+            replication=replication,
         )
 
     def create_key(
@@ -170,9 +184,15 @@ class B2Session(object):
     def delete_file_version(self, file_id, file_name):
         return self._wrap_default_token(self.raw_api.delete_file_version, file_id, file_name)
 
-    def download_file_from_url(self, url, range_=None):
+    def download_file_from_url(
+        self, url, range_=None, encryption: Optional[EncryptionSetting] = None
+    ):
         return self._wrap_token(
-            self.raw_api.download_file_from_url, TokenType.API_TOKEN_ONLY, url, range_=range_
+            self.raw_api.download_file_from_url,
+            TokenType.API_TOKEN_ONLY,
+            url,
+            range_=range_,
+            encryption=encryption,
         )
 
     def finish_large_file(self, file_id, part_sha1_array):
@@ -267,9 +287,25 @@ class B2Session(object):
             prefix=prefix,
         )
 
-    def start_large_file(self, bucket_id, file_name, content_type, file_info):
+    def start_large_file(
+        self,
+        bucket_id,
+        file_name,
+        content_type,
+        file_info,
+        server_side_encryption: Optional[EncryptionSetting] = None,
+        file_retention: Optional[FileRetentionSetting] = None,
+        legal_hold: Optional[LegalHold] = None,
+    ):
         return self._wrap_default_token(
-            self.raw_api.start_large_file, bucket_id, file_name, content_type, file_info
+            self.raw_api.start_large_file,
+            bucket_id,
+            file_name,
+            content_type,
+            file_info,
+            server_side_encryption,
+            file_retention=file_retention,
+            legal_hold=legal_hold,
         )
 
     def update_bucket(
@@ -281,6 +317,9 @@ class B2Session(object):
         cors_rules=None,
         lifecycle_rules=None,
         if_revision_is=None,
+        default_server_side_encryption: Optional[EncryptionSetting] = None,
+        default_retention: Optional[BucketRetentionSetting] = None,
+        replication: Optional[ReplicationConfiguration] = None,
     ):
         return self._wrap_default_token(
             self.raw_api.update_bucket,
@@ -291,11 +330,23 @@ class B2Session(object):
             cors_rules=cors_rules,
             lifecycle_rules=lifecycle_rules,
             if_revision_is=if_revision_is,
+            default_server_side_encryption=default_server_side_encryption,
+            default_retention=default_retention,
+            replication=replication,
         )
 
     def upload_file(
-        self, bucket_id, file_name, content_length, content_type, content_sha1, file_infos,
-        data_stream
+        self,
+        bucket_id,
+        file_name,
+        content_length,
+        content_type,
+        content_sha1,
+        file_infos,
+        data_stream,
+        server_side_encryption: Optional[EncryptionSetting] = None,
+        file_retention: Optional[FileRetentionSetting] = None,
+        legal_hold: Optional[LegalHold] = None,
     ):
         return self._wrap_token(
             self.raw_api.upload_file,
@@ -307,9 +358,20 @@ class B2Session(object):
             content_sha1,
             file_infos,
             data_stream,
+            server_side_encryption,
+            file_retention=file_retention,
+            legal_hold=legal_hold,
         )
 
-    def upload_part(self, file_id, part_number, content_length, sha1_sum, input_stream):
+    def upload_part(
+        self,
+        file_id,
+        part_number,
+        content_length,
+        sha1_sum,
+        input_stream,
+        server_side_encryption: Optional[EncryptionSetting] = None,
+    ):
         return self._wrap_token(
             self.raw_api.upload_part,
             TokenType.UPLOAD_PART,
@@ -318,6 +380,7 @@ class B2Session(object):
             content_length,
             sha1_sum,
             input_stream,
+            server_side_encryption,
         )
 
     def get_download_url_by_id(self, file_id):
@@ -337,6 +400,10 @@ class B2Session(object):
         content_type=None,
         file_info=None,
         destination_bucket_id=None,
+        destination_server_side_encryption: Optional[EncryptionSetting] = None,
+        source_server_side_encryption: Optional[EncryptionSetting] = None,
+        file_retention: Optional[FileRetentionSetting] = None,
+        legal_hold: Optional[LegalHold] = None,
     ):
         return self._wrap_default_token(
             self.raw_api.copy_file,
@@ -347,6 +414,10 @@ class B2Session(object):
             content_type=content_type,
             file_info=file_info,
             destination_bucket_id=destination_bucket_id,
+            destination_server_side_encryption=destination_server_side_encryption,
+            source_server_side_encryption=source_server_side_encryption,
+            file_retention=file_retention,
+            legal_hold=legal_hold,
         )
 
     def copy_part(
@@ -355,13 +426,17 @@ class B2Session(object):
         large_file_id,
         part_number,
         bytes_range=None,
+        destination_server_side_encryption: Optional[EncryptionSetting] = None,
+        source_server_side_encryption: Optional[EncryptionSetting] = None,
     ):
         return self._wrap_default_token(
             self.raw_api.copy_part,
             source_file_id,
             large_file_id,
             part_number,
-            bytes_range=bytes_range
+            bytes_range=bytes_range,
+            destination_server_side_encryption=destination_server_side_encryption,
+            source_server_side_encryption=source_server_side_encryption,
         )
 
     def _wrap_default_token(self, raw_api_method, *args, **kwargs):
@@ -464,3 +539,31 @@ class B2Session(object):
         response = f(upload_url, upload_auth_token, *args, **kwargs)
         self.account_info.put_large_file_upload_url(file_id, upload_url, upload_auth_token)
         return response
+
+    def update_file_retention(
+        self,
+        file_id,
+        file_name,
+        file_retention: FileRetentionSetting,
+        bypass_governance: bool = False,
+    ):
+        return self._wrap_default_token(
+            self.raw_api.update_file_retention,
+            file_id,
+            file_name,
+            file_retention,
+            bypass_governance,
+        )
+
+    def update_file_legal_hold(
+        self,
+        file_id,
+        file_name,
+        legal_hold: LegalHold,
+    ):
+        return self._wrap_default_token(
+            self.raw_api.update_file_legal_hold,
+            file_id,
+            file_name,
+            legal_hold,
+        )
