@@ -8,6 +8,7 @@
 #
 ######################################################################
 
+from math import ceil
 import hashlib
 import json
 
@@ -29,7 +30,7 @@ MEGABYTE = 1000 * 1000
 GIGABYTE = 1000 * MEGABYTE
 
 
-class UploadBuffer(object):
+class UploadBuffer:
     """ data container used by EmergePlanner for temporary storage of write intents """
 
     def __init__(self, start_offset, buff=None):
@@ -79,7 +80,7 @@ class UploadBuffer(object):
         return self.__class__(start_offset, buff_slice)
 
 
-class EmergePlanner(object):
+class EmergePlanner:
     """ Creates a list of actions required for advanced creation of an object in the cloud from an iterator of write intent objects """
     DEFAULT_MIN_PART_SIZE = 5 * MEGABYTE
     DEFAULT_RECOMMENDED_UPLOAD_PART_SIZE = 100 * MEGABYTE
@@ -104,10 +105,8 @@ class EmergePlanner(object):
         recommended_upload_part_size=None,
         max_part_size=None
     ):
-        # TODO: add support for getting `min_part_size` and `max_part_size` from account info
         if recommended_upload_part_size is None:
-            # TODO: change `get_minimum_part_size` to correct name
-            recommended_upload_part_size = account_info.get_minimum_part_size()
+            recommended_upload_part_size = account_info.get_recommended_part_size()
         if min_part_size is None and recommended_upload_part_size < cls.DEFAULT_MIN_PART_SIZE:
             min_part_size = recommended_upload_part_size
         if max_part_size is None and recommended_upload_part_size > cls.DEFAULT_MAX_PART_SIZE:
@@ -121,6 +120,22 @@ class EmergePlanner(object):
 
     def get_emerge_plan(self, write_intents):
         write_intents = sorted(write_intents, key=lambda intent: intent.destination_offset)
+
+        # the upload part size recommended by the server causes errors with files larger than 1TB
+        # (with the current 100MB part size and 10000 part count limit).
+        # Therefore here we increase the recommended upload part size if needed.
+        # the constant is for handling mixed upload/copy in concatenate etc
+        max_destination_offset = max(intent.destination_end_offset for intent in write_intents)
+        self.recommended_upload_part_size = max(
+            self.recommended_upload_part_size,
+            min(
+                ceil(1.5 * max_destination_offset / 10000),
+                self.max_part_size,
+            )
+        )
+        assert self.min_part_size <= self.recommended_upload_part_size <= self.max_part_size, (
+            self.min_part_size, self.recommended_upload_part_size, self.max_part_size
+        )
         return self._get_emerge_plan(write_intents, EmergePlan)
 
     def get_streaming_emerge_plan(self, write_intent_iterator):
@@ -168,7 +183,7 @@ class EmergePlanner(object):
             # if this is a copy intent and we want to copy it server-side, then we have to
             # flush the whole upload buffer we accumulated so far, but OTOH we may decide that we just want to
             # append it to upload buffer (see complete, untrivial logic below) and then maybe
-            # flush some upload parts from upload bufffer (if there is enough in the buffer)
+            # flush some upload parts from upload buffer (if there is enough in the buffer)
 
             current_len = current_end - upload_buffer.end_offset
             # should we flush the upload buffer or do we have to add a chunk of the copy first?
@@ -293,7 +308,7 @@ class EmergePlanner(object):
             if tail_buffer.length < self.recommended_upload_part_size + self.min_part_size:
                 # `EmergePlanner_buff_partition` can split in such way that tail part
                 # can be smaller than `min_part_size` - to avoid unnecessary download of possible
-                # incoming copy intent, we don't split futher
+                # incoming copy intent, we don't split further
                 yield tail_buffer
                 return
             head_buff, tail_buffer = self._buff_partition(tail_buffer)
@@ -302,7 +317,7 @@ class EmergePlanner(object):
     def _buff_partition(self, upload_buffer):
         """ Split upload buffer to two parts (smaller upload buffers).
 
-        In result left part cannot be splitted more, and nothing can be assumed about right part.
+        In result left part cannot be split more, and nothing can be assumed about right part.
 
         :rtype tuple(b2sdk.transfer.emerge.planner.planner.UploadBuffer,
                      b2sdk.transfer.emerge.planner.planner.UploadBuffer):
@@ -333,7 +348,7 @@ class EmergePlanner(object):
         would be merged again by higher level iterator that produces emerge parts, but
         in principle this merging can happen here. Not merging it is a code design decision
         to make this function easier to implement and also it would allow yielding emerge parts
-        a bit quickier.
+        a bit quicker.
         """
 
         # `protected_intent_length` for upload state is 0, so it would generate at most single intent fragment
@@ -386,7 +401,7 @@ class EmergePlanner(object):
     def _merge_intent_fragments(self, start_offset, upload_intents, copy_intents):
         """ Select "competing" upload and copy fragments.
 
-        Upload and copy fragments may overlap so we nedd to choose right one
+        Upload and copy fragments may overlap so we need to choose right one
         to use - copy fragments are prioritized unless this fragment is unprotected
         (we use "protection" as an abstract for "short copy" fragments - meaning upload
         fragments have higher priority than "short copy")
@@ -432,7 +447,7 @@ class EmergePlanner(object):
             yield write_intent
 
 
-class IntentsState(object):
+class IntentsState:
     """ Store and process state of incoming write intents to solve
     overlapping intents selection in streaming manner.
 
@@ -477,7 +492,7 @@ class IntentsState(object):
         would not be added to this intents state. It would yield a state of this stream
         of intents (like copy or upload) from ``last_sent_offset`` to ``incoming_offset``.
         So here happens the first stage of solving overlapping intents selection - but
-        write intent iterator can be splitted to multiple substreams (like copy and upload)
+        write intent iterator can be split to multiple substreams (like copy and upload)
         so additional stage is required to cover this.
         """
         if self._current_intent is not None:
@@ -560,7 +575,7 @@ class IntentsState(object):
         we need to know for fragment if it is a "small copy" or not. In result of solving
         overlapping intents selection there might be a situation when original intent was not
         a small copy, but in effect it will be used only partially and in effect it may be a "small copy".
-        Algorithm attempts to aviod using smaller fragments than ``protected_intent_length`` but
+        Algorithm attempts to avoid using smaller fragments than ``protected_intent_length`` but
         sometimes it may be impossible. So if this function returns ``False`` it means
         that used length of this intent is smaller than ``protected_intent_length`` and the algorithm
         was unable to avoid this.
@@ -636,7 +651,7 @@ class StreamingEmergePlan(BaseEmergePlan):
             return chain([first_part, second_part], emerge_parts_iterator), True
 
 
-class EmergePart(object):
+class EmergePart:
     def __init__(self, part_definition, verification_ranges=None):
         self.part_definition = part_definition
         self.verification_ranges = verification_ranges
