@@ -10,17 +10,13 @@
 
 B2Error = None  # calm down, pyflakes
 
-from b2sdk.v1.exception import *
+from b2sdk.v1.exception import *  # noqa
+
+v1DestFileNewer = DestFileNewer
 
 
-class DestFileNewer(B2Error):
-    def __init__(self, dest_file, source_file, dest_prefix, source_prefix):
-        super(DestFileNewer, self).__init__()
-        self.dest_file = dest_file
-        self.source_file = source_file
-        self.dest_prefix = dest_prefix
-        self.source_prefix = source_prefix
-
+# override to retain old style __str__
+class DestFileNewer(v1DestFileNewer):
     def __str__(self):
         return 'source file is older than destination: %s%s with a time of %s cannot be synced to %s%s with a time of %s, unless --skipNewer or --replaceNewer is provided' % (
             self.source_prefix,
@@ -30,6 +26,3 @@ class DestFileNewer(B2Error):
             self.dest_file.name,
             self.dest_file.latest_version().mod_time,
         )
-
-    def should_retry_http(self):
-        return True
