@@ -17,6 +17,9 @@ import pytest
 def test_files_headers():
     for file in glob('**/*.py', recursive=True):
         with open(file) as fd:
+            file = file.replace(
+                '\\', '/'
+            )  # glob('**/*.py') on Windows returns "b2\bucket.py" (wrong slash)
             head = ''.join(islice(fd, 9))
             if 'All Rights Reserved' not in head:
                 pytest.fail('Missing "All Rights Reserved" in the header in: {}'.format(file))
