@@ -8,11 +8,14 @@
 #
 ######################################################################
 
+import pytest
+
 from apiver_deps_exception import (
     EnvironmentEncodingError,
     InvalidArgument,
     IncompleteSync,
-    UnSyncableFilename,
+    UnsupportedFilename,
+    check_invalid_argument,
 )
 
 
@@ -38,8 +41,28 @@ export LANG=en_US.UTF-8""", str(e)
         except IncompleteSync as e:
             assert str(e) == 'Incomplete sync: ', str(e)
 
-    def test_unsyncablefilename_error(self):
+    def test_unsupportedfilename_error(self):
         try:
-            raise UnSyncableFilename('message', 'filename')
-        except UnSyncableFilename as e:
+            raise UnsupportedFilename('message', 'filename')
+        except UnsupportedFilename as e:
             assert str(e) == 'message: filename', str(e)
+
+
+class TestCheckInvalidArgument:
+    def test_custom_message(self):
+        with pytest.raises(InvalidArgument):
+            try:
+                with check_invalid_argument('param', 'an error occurred', RuntimeError):
+                    raise RuntimeError()
+            except InvalidArgument as exc:
+                assert str(exc) == 'param an error occurred'
+                raise
+
+    def test_message_from_exception(self):
+        with pytest.raises(InvalidArgument):
+            try:
+                with check_invalid_argument('param', '', RuntimeError):
+                    raise RuntimeError('an error occurred')
+            except InvalidArgument as exc:
+                assert str(exc) == 'param an error occurred'
+                raise
