@@ -2,33 +2,44 @@
 #
 # File: test/unit/sync/test_sync.py
 #
-# Copyright 2020 Backblaze Inc. All Rights Reserved.
+# Copyright 2022 Backblaze Inc. All Rights Reserved.
 #
 # License https://www.backblaze.com/using_b2_code.html
 #
 ######################################################################
-
+from collections import defaultdict
+from unittest import mock
 from enum import Enum
 from functools import partial
 
-from apiver_deps_exception import CommandError, DestFileNewer, InvalidArgument
-
+from apiver_deps import UpPolicy, B2DownloadAction, AbstractSyncEncryptionSettingsProvider, UploadSourceLocalFile, SyncPolicyManager
+from apiver_deps_exception import DestFileNewer, InvalidArgument
+from apiver_deps import KeepOrDeleteMode, NewerFileSyncMode, CompareVersionMode
+import pytest
+from ..fixtures.folder import *
 from .fixtures import *
 
 DAY = 86400000  # milliseconds
 TODAY = DAY * 100  # an arbitrary reference time for testing
 
 
-class TestSynchronizer:
-    class IllegalEnum(Enum):
-        ILLEGAL = 5100
+class IllegalEnum(Enum):
+    ILLEGAL = 5100
 
+
+class TestSynchronizer:
     @pytest.fixture(autouse=True)
-    def setup(self, folder_factory, mocker):
+    def setup(self, folder_factory, mocker, apiver):
         self.folder_factory = folder_factory
         self.local_folder_factory = partial(folder_factory, 'local')
         self.b2_folder_factory = partial(folder_factory, 'b2')
         self.reporter = mocker.MagicMock()
+        self.apiver = apiver
+
+    def _make_folder_sync_actions(self, synchronizer, *args, **kwargs):
+        if self.apiver in ['v0', 'v1']:
+            return synchronizer.make_folder_sync_actions(*args, **kwargs)
+        return synchronizer._make_folder_sync_actions(*args, **kwargs)
 
     def assert_folder_sync_actions(self, synchronizer, src_folder, dst_folder, expected_actions):
         """
@@ -37,8 +48,10 @@ class TestSynchronizer:
 
         The source and destination files may have multiple versions.
         """
+
         actions = list(
-            synchronizer.make_folder_sync_actions(
+            self._make_folder_sync_actions(
+                synchronizer,
                 src_folder,
                 dst_folder,
                 TODAY,
@@ -47,6 +60,7 @@ class TestSynchronizer:
         )
         assert expected_actions == [str(a) for a in actions]
 
+    @pytest.mark.apiver(to_ver=0)
     @pytest.mark.parametrize(
         'args', [
             {
@@ -61,13 +75,28 @@ class TestSynchronizer:
             'keep_days_or_delete',
         ]
     )
-    def test_illegal_args(self, synchronizer_factory, apiver, args):
-        exceptions = {
-            'v1': InvalidArgument,
-            'v0': CommandError,
-        }
+    def test_illegal_args_up_to_v0(self, synchronizer_factory, apiver, args):
+        from apiver_deps_exception import CommandError
+        with pytest.raises(CommandError):
+            synchronizer_factory(**args)
 
-        with pytest.raises(exceptions[apiver]):
+    @pytest.mark.apiver(from_ver=1)
+    @pytest.mark.parametrize(
+        'args', [
+            {
+                'newer_file_mode': IllegalEnum.ILLEGAL
+            },
+            {
+                'keep_days_or_delete': IllegalEnum.ILLEGAL
+            },
+        ],
+        ids=[
+            'newer_file_mode',
+            'keep_days_or_delete',
+        ]
+    )
+    def test_illegal_args_up_v1_and_up(self, synchronizer_factory, apiver, args):
+        with pytest.raises(InvalidArgument):
             synchronizer_factory(**args)
 
     def test_illegal(self, synchronizer):
@@ -516,14 +545,14 @@ class TestSynchronizer:
         dst = self.folder_factory(dst_type, ('a.txt', [200]))
         with pytest.raises(DestFileNewer) as excinfo:
             self.assert_folder_sync_actions(synchronizer, src, dst, expected)
-        messages = {
-            'v1': 'source file is older than destination: %s://a.txt with a time of 100 '
-                  'cannot be synced to %s://a.txt with a time of 200, '
-                  'unless a valid newer_file_mode is provided',
-            'v0': 'source file is older than destination: %s://a.txt with a time of 100 '
-                  'cannot be synced to %s://a.txt with a time of 200, '
-                  'unless --skipNewer or --replaceNewer is provided',
-        }  # yapf: disable
+        messages = defaultdict(
+            lambda: 'source file is older than destination: %s://a.txt with a time of 100 '
+                    'cannot be synced to %s://a.txt with a time of 200, '
+                    'unless a valid newer_file_mode is provided',
+            v0='source file is older than destination: %s://a.txt with a time of 100 '
+               'cannot be synced to %s://a.txt with a time of 200, '
+               'unless --skipNewer or --replaceNewer is provided'
+        )  # yapf: disable
 
         assert str(excinfo.value) == messages[apiver] % (src_type, dst_type)
 
@@ -666,3 +695,205 @@ class TestSynchronizer:
         src = self.folder_factory(src_type, ('a.txt', [200], 11))
         dst = self.folder_factory(dst_type, ('a.txt', [100], 10))
         self.assert_folder_sync_actions(synchronizer, src, dst, expected)
+
+    # FIXME: rewrite this test to not use mock.call checks when all of Synchronizers tests are rewritten to test_bucket
+    # style - i.e. with simulated api and fake files returned from methods. Then, checking EncryptionSetting used for
+    # transmission will be done by the underlying simulator.
+    def test_encryption_b2_to_local(self, synchronizer_factory, apiver):
+        local = self.local_folder_factory()
+        remote = self.b2_folder_factory(('directory/b.txt', [100]))
+        synchronizer = synchronizer_factory()
+
+        encryption = object()
+        bucket = mock.MagicMock()
+        provider = TstEncryptionSettingsProvider(encryption, encryption)
+        download_action = next(
+            iter(
+                self._make_folder_sync_actions(
+                    synchronizer,
+                    remote,
+                    local,
+                    TODAY,
+                    self.reporter,
+                    encryption_settings_provider=provider
+                )
+            )
+        )
+        with mock.patch.object(B2DownloadAction, '_ensure_directory_existence'):
+            try:
+                download_action.do_action(bucket, self.reporter)
+            except:
+                pass
+
+        assert bucket.mock_calls[0] == mock.call.download_file_by_id(
+            'id_d_100', progress_listener=mock.ANY, encryption=encryption
+        )
+
+        if apiver in ['v0', 'v1']:
+            file_version_kwarg = 'file_version_info'
+        else:
+            file_version_kwarg = 'file_version'
+
+        provider.get_setting_for_download.assert_has_calls(
+            [mock.call(
+                bucket=bucket,
+                **{file_version_kwarg: mock.ANY},
+            )]
+        )
+
+    # FIXME: rewrite this test to not use mock.call checks when all of Synchronizers tests are rewritten to test_bucket
+    # style - i.e. with simulated api and fake files returned from methods. Then, checking EncryptionSetting used for
+    # transmission will be done by the underlying simulator.
+    def test_encryption_local_to_b2(self, synchronizer_factory):
+        local = self.local_folder_factory(('directory/a.txt', [100]))
+        remote = self.b2_folder_factory()
+        synchronizer = synchronizer_factory()
+
+        encryption = object()
+        bucket = mock.MagicMock()
+        provider = TstEncryptionSettingsProvider(encryption, encryption)
+        upload_action = next(
+            iter(
+                self._make_folder_sync_actions(
+                    synchronizer,
+                    local,
+                    remote,
+                    TODAY,
+                    self.reporter,
+                    encryption_settings_provider=provider
+                )
+            )
+        )
+        with mock.patch.object(UploadSourceLocalFile, 'check_path_and_get_size'):
+            try:
+                upload_action.do_action(bucket, self.reporter)
+            except:
+                pass
+
+        assert bucket.mock_calls == [
+            mock.call.upload(
+                mock.ANY,
+                'folder/directory/a.txt',
+                file_info={'src_last_modified_millis': '100'},
+                progress_listener=mock.ANY,
+                encryption=encryption
+            )
+        ]
+
+        assert provider.get_setting_for_upload.mock_calls == [
+            mock.call(
+                bucket=bucket,
+                b2_file_name='folder/directory/a.txt',
+                file_info={'src_last_modified_millis': '100'},
+                length=10
+            )
+        ]
+
+    # FIXME: rewrite this test to not use mock.call checks when all of Synchronizers tests are rewritten to test_bucket
+    # style - i.e. with simulated api and fake files returned from methods. Then, checking EncryptionSetting used for
+    # transmission will be done by the underlying simulator.
+    def test_encryption_b2_to_b2(self, synchronizer_factory, apiver):
+        src = self.b2_folder_factory(('directory/a.txt', [100]))
+        dst = self.b2_folder_factory()
+        synchronizer = synchronizer_factory()
+
+        source_encryption = object()
+        destination_encryption = object()
+        bucket = mock.MagicMock()
+        provider = TstEncryptionSettingsProvider(source_encryption, destination_encryption)
+        copy_action = next(
+            iter(
+                self._make_folder_sync_actions(
+                    synchronizer,
+                    src,
+                    dst,
+                    TODAY,
+                    self.reporter,
+                    encryption_settings_provider=provider
+                )
+            )
+        )
+        copy_action.do_action(bucket, self.reporter)
+
+        assert bucket.mock_calls == [
+            mock.call.copy(
+                'id_d_100',
+                'folder/directory/a.txt',
+                length=10,
+                source_content_type='text/plain',
+                source_file_info={'in_b2': 'yes'},
+                progress_listener=mock.ANY,
+                source_encryption=source_encryption,
+                destination_encryption=destination_encryption
+            )
+        ]
+
+        if apiver in ['v0', 'v1']:
+            file_version_kwarg = 'source_file_version_info'
+            additional_kwargs = {'target_file_info': None}
+        else:
+            file_version_kwarg = 'source_file_version'
+            additional_kwargs = {}
+
+        assert provider.get_source_setting_for_copy.mock_calls == [
+            mock.call(
+                bucket=mock.ANY,
+                **{file_version_kwarg: mock.ANY},
+            )
+        ]
+
+        assert provider.get_destination_setting_for_copy.mock_calls == [
+            mock.call(
+                bucket=mock.ANY,
+                dest_b2_file_name='folder/directory/a.txt',
+                **additional_kwargs,
+                **{file_version_kwarg: mock.ANY},
+            )
+        ]
+
+    def test_custom_sync_manager_policy(self, synchronizer_factory):
+        class MySyncPolicyManager(SyncPolicyManager):
+            def get_policy_class(self, sync_type, delete, keep_days):
+                return UpPolicy
+
+        synchronizer = synchronizer_factory(
+            compare_version_mode=CompareVersionMode.SIZE,
+            keep_days_or_delete=KeepOrDeleteMode.DELETE,
+            sync_policy_manager=MySyncPolicyManager(),
+        )
+        src = self.folder_factory('local', ('a.txt', [200], 11))
+        dst = self.folder_factory('b2', ('a.txt', [100], 10))
+        # normally_expected = [
+        #     'b2_upload(/dir/a.txt, folder/a.txt, 200)',
+        #     'b2_delete(folder/a.txt, id_a_100, (old version))'
+        # ]
+        expected = ['b2_upload(/dir/a.txt, folder/a.txt, 200)']
+        self.assert_folder_sync_actions(synchronizer, src, dst, expected)
+
+
+class TstEncryptionSettingsProvider(AbstractSyncEncryptionSettingsProvider):
+    def __init__(self, source_encryption_setting, destination_encryption_setting):
+        self.get_setting_for_upload = mock.MagicMock(
+            side_effect=lambda *a, **kw: destination_encryption_setting
+        )
+        self.get_source_setting_for_copy = mock.MagicMock(
+            side_effect=lambda *a, **kw: source_encryption_setting
+        )
+        self.get_destination_setting_for_copy = mock.MagicMock(
+            side_effect=lambda *a, **kw: destination_encryption_setting
+        )
+        self.get_setting_for_download = mock.MagicMock(
+            side_effect=lambda *a, **kw: source_encryption_setting
+        )
+
+    def get_setting_for_upload(self, *a, **kw):
+        """overwritten in __init__"""
+
+    def get_source_setting_for_copy(self, *a, **kw):
+        """overwritten in __init__"""
+
+    def get_destination_setting_for_copy(self, *a, **kw):
+        """overwritten in __init__"""
+
+    def get_setting_for_download(self, *a, **kw):
+        """overwritten in __init__"""
