@@ -8,4 +8,6 @@
 #
 ######################################################################
 
-from b2sdk.v0 import *
+from b2sdk.v0 import *  # noqa
+
+V = 0
