@@ -15,7 +15,7 @@ import unittest.mock as mock
 
 import pytest
 
-from .test_base import TestBase
+from ..test_base import TestBase
 
 from .deps_exception import (
     AlreadyFailed,
@@ -26,19 +26,50 @@ from .deps_exception import (
     InvalidUploadSource,
     MaxRetriesExceeded,
     UnsatisfiableRange,
+    SSECKeyError,
 )
 from .deps import B2Api
 from .deps import LargeFileUploadState
 from .deps import DownloadDestBytes, PreSeekedDownloadDest
 from .deps import FileVersionInfo
+from .deps import LegalHold, FileRetentionSetting, RetentionMode, NO_RETENTION_FILE_SETTING
 from .deps import MetadataDirectiveMode
 from .deps import Part
 from .deps import AbstractProgressListener
-from .deps import StubAccountInfo, RawSimulator, BucketSimulator, FakeResponse
+from .deps import StubAccountInfo, RawSimulator, BucketSimulator, FakeResponse, FileSimulator
 from .deps import ParallelDownloader
 from .deps import SimpleDownloader
 from .deps import UploadSourceBytes
 from .deps import hex_sha1_of_bytes, TempDir
+from .deps import EncryptionAlgorithm, EncryptionSetting, EncryptionSettingFactory, EncryptionMode, \
+    EncryptionKey, SSE_NONE, SSE_B2_AES
+from .deps import CopySource, UploadSourceLocalFile, WriteIntent
+
+SSE_C_AES = EncryptionSetting(
+    mode=EncryptionMode.SSE_C,
+    algorithm=EncryptionAlgorithm.AES256,
+    key=EncryptionKey(secret=b'some_key', key_id='some-id'),
+)
+SSE_C_AES_NO_SECRET = EncryptionSetting(
+    mode=EncryptionMode.SSE_C,
+    algorithm=EncryptionAlgorithm.AES256,
+    key=EncryptionKey(secret=None, key_id='some-id'),
+)
+SSE_C_AES_2 = EncryptionSetting(
+    mode=EncryptionMode.SSE_C,
+    algorithm=EncryptionAlgorithm.AES256,
+    key=EncryptionKey(secret=b'some_other_key', key_id='some-id-2'),
+)
+SSE_C_AES_2_NO_SECRET = EncryptionSetting(
+    mode=EncryptionMode.SSE_C,
+    algorithm=EncryptionAlgorithm.AES256,
+    key=EncryptionKey(secret=None, key_id='some-id-2'),
+)
+SSE_C_AES_FROM_SERVER = EncryptionSetting(
+    mode=EncryptionMode.SSE_C,
+    algorithm=EncryptionAlgorithm.AES256,
+    key=EncryptionKey(key_id=None, secret=None),
+)
 
 
 def write_file(path, data):
@@ -143,10 +174,26 @@ class TestCaseWithBucket(TestBase):
         ]
         self.assertEqual(expected, actual)
 
+    def _make_data(self, approximate_length):
+        """
+        Generate a sequence of bytes to use in testing an upload.
+        Don't repeat a short pattern, so we're sure that the different
+        parts of a large file are actually different.
+
+        Returns bytes.
+        """
+        fragments = []
+        so_far = 0
+        while so_far < approximate_length:
+            fragment = ('%d:' % so_far).encode('utf-8')
+            so_far += len(fragment)
+            fragments.append(fragment)
+        return b''.join(fragments)
+
 
 class TestReauthorization(TestCaseWithBucket):
     def testCreateBucket(self):
-        class InvalidAuthTokenWrapper(object):
+        class InvalidAuthTokenWrapper:
             def __init__(self, original_function):
                 self.__original_function = original_function
                 self.__name__ = original_function.__name__
@@ -235,8 +282,56 @@ class TestGetFileInfo(TestCaseWithBucket):
         info = self.bucket.get_file_info_by_name('a')
 
         self.assertIsInstance(info, FileVersionInfo)
-        expected = (a_id, 'a', 11, None, 'b2/x-auto')
-        actual = (info.id_, info.file_name, info.size, info.action, info.content_type)
+        expected = (
+            a_id, 'a', 11, 'upload', 'b2/x-auto', 'none', NO_RETENTION_FILE_SETTING, LegalHold.UNSET
+        )
+        actual = (
+            info.id_,
+            info.file_name,
+            info.size,
+            info.action,
+            info.content_type,
+            info.server_side_encryption.mode.value,
+            info.file_retention,
+            info.legal_hold,
+        )
+        self.assertEqual(expected, actual)
+
+    def test_version_by_name_file_lock(self):
+        bucket = self.api.create_bucket(
+            'my-bucket-with-file-lock', 'allPublic', is_file_lock_enabled=True
+        )
+        data = b'hello world'
+        legal_hold = LegalHold.ON
+        file_retention = FileRetentionSetting(RetentionMode.COMPLIANCE, 100)
+
+        bucket.upload_bytes(data, 'a', file_retention=file_retention, legal_hold=legal_hold)
+
+        file_version = bucket.get_file_info_by_name('a')
+
+        actual = (file_version.legal_hold, file_version.file_retention)
+        self.assertEqual((legal_hold, file_retention), actual)
+
+        low_perm_account_info = StubAccountInfo()
+        low_perm_api = B2Api(low_perm_account_info, raw_api=self.simulator)
+        low_perm_key_resp = self.api.create_key(
+            key_name='lowperm', capabilities=[
+                'listKeys',
+                'listBuckets',
+                'listFiles',
+                'readFiles',
+            ]
+        )
+
+        low_perm_api.authorize_account(
+            'production', low_perm_key_resp['applicationKeyId'], low_perm_key_resp['applicationKey']
+        )
+        low_perm_bucket = low_perm_api.get_bucket_by_name('my-bucket-with-file-lock')
+
+        file_version = low_perm_bucket.get_file_info_by_name('a')
+
+        actual = (file_version.legal_hold, file_version.file_retention)
+        expected = (LegalHold.UNKNOWN, FileRetentionSetting(RetentionMode.UNKNOWN))
         self.assertEqual(expected, actual)
 
     def test_version_by_id(self):
@@ -335,17 +430,53 @@ class TestLs(TestCaseWithBucket):
         self.assertBucketContents(expected, '', show_versions=True)
 
 
+class TestListVersions(TestCaseWithBucket):
+    def test_encryption(self):
+        data = b'hello world'
+        a = self.bucket.upload_bytes(data, 'a')
+        a_id = a.id_
+        self.assertEqual(a.server_side_encryption, SSE_NONE)
+        b = self.bucket.upload_bytes(data, 'b', encryption=SSE_B2_AES)
+        self.assertEqual(b.server_side_encryption, SSE_B2_AES)
+        b_id = b.id_
+        # c_id = self.bucket.upload_bytes(data, 'c', encryption=SSE_NONE).id_  # TODO
+        self.bucket.copy(a_id, 'd', destination_encryption=SSE_B2_AES)
+        self.bucket.copy(
+            b_id, 'e', destination_encryption=SSE_C_AES, file_info={}, content_type='text/plain'
+        )
+
+        actual = [info for info in self.bucket.list_file_versions('a')['files']][0]
+        actual = EncryptionSettingFactory.from_file_version_dict(actual)
+        self.assertEqual(SSE_NONE, actual)  # bucket default
+
+        actual = [info for info in self.bucket.list_file_versions('b')['files']][0]
+        actual = EncryptionSettingFactory.from_file_version_dict(actual)
+        self.assertEqual(SSE_B2_AES, actual)  # explicitly requested sse-b2
+
+        # actual = [info for info in self.bucket.list_file_versions('c')][0]
+        # actual = EncryptionSettingFactory.from_file_version_dict(actual)
+        # self.assertEqual(SSE_NONE, actual)  # explicitly requested none
+
+        actual = [info for info in self.bucket.list_file_versions('d')['files']][0]
+        actual = EncryptionSettingFactory.from_file_version_dict(actual)
+        self.assertEqual(SSE_B2_AES, actual)  # explicitly requested sse-b2
+
+        actual = [info for info in self.bucket.list_file_versions('e')['files']][0]
+        actual = EncryptionSettingFactory.from_file_version_dict(actual)
+        self.assertEqual(SSE_C_AES_NO_SECRET, actual)  # explicitly requested sse-c
+
+
 class TestCopyFile(TestCaseWithBucket):
     def test_copy_without_optional_params(self):
         file_id = self._make_file()
         self.bucket.copy_file(file_id, 'hello_new.txt')
-        expected = [('hello.txt', 11, 'upload', None), ('hello_new.txt', 11, 'copy', None)]
+        expected = [('hello.txt', 11, 'upload', None), ('hello_new.txt', 11, 'upload', None)]
         self.assertBucketContents(expected, '', show_versions=True)
 
     def test_copy_with_range(self):
         file_id = self._make_file()
         self.bucket.copy_file(file_id, 'hello_new.txt', bytes_range=(3, 9))
-        expected = [('hello.txt', 11, 'upload', None), ('hello_new.txt', 6, 'copy', None)]
+        expected = [('hello.txt', 11, 'upload', None), ('hello_new.txt', 7, 'upload', None)]
         self.assertBucketContents(expected, '', show_versions=True)
 
     def test_copy_with_invalid_metadata(self):
@@ -393,7 +524,7 @@ class TestCopyFile(TestCaseWithBucket):
         )
         expected = [
             ('hello.txt', 11, 'upload', 'b2/x-auto', None),
-            ('hello_new.txt', 11, 'copy', 'text/plain', None),
+            ('hello_new.txt', 11, 'upload', 'text/plain', None),
         ]
         actual = [
             (info.file_name, info.size, info.action, info.content_type, folder)
@@ -431,9 +562,150 @@ class TestCopyFile(TestCaseWithBucket):
 
         expected = [('hello.txt', 11, 'upload', None)]
         self.assertEqual(expected, ls(source_bucket))
-        expected = [('hello_new.txt', 11, 'copy', None)]
+        expected = [('hello_new.txt', 11, 'upload', None)]
         self.assertBucketContents(expected, '', show_versions=True)
 
+    def test_copy_retention(self):
+        for data in [self._make_data(self.simulator.MIN_PART_SIZE * 3), b'hello']:
+            for length in [None, len(data)]:
+                with self.subTest(real_length=len(data), length=length):
+                    file_id = self.bucket.upload_bytes(data, 'original_file').id_
+                    resulting_file_version = self.bucket.copy(
+                        file_id,
+                        'copied_file',
+                        file_retention=FileRetentionSetting(RetentionMode.COMPLIANCE, 100),
+                        legal_hold=LegalHold.ON,
+                        max_part_size=400,
+                    )
+                    self.assertEqual(
+                        FileRetentionSetting(RetentionMode.COMPLIANCE, 100),
+                        resulting_file_version.file_retention
+                    )
+                    self.assertEqual(LegalHold.ON, resulting_file_version.legal_hold)
+
+    def test_copy_encryption(self):
+        data = b'hello_world'
+        a = self.bucket.upload_bytes(data, 'a')
+        a_id = a.id_
+        self.assertEqual(a.server_side_encryption, SSE_NONE)
+
+        b = self.bucket.upload_bytes(data, 'b', encryption=SSE_B2_AES)
+        self.assertEqual(b.server_side_encryption, SSE_B2_AES)
+        b_id = b.id_
+
+        c = self.bucket.upload_bytes(data, 'c', encryption=SSE_C_AES)
+        self.assertEqual(c.server_side_encryption, SSE_C_AES_NO_SECRET)
+        c_id = c.id_
+
+        for length in [None, len(data)]:
+            for kwargs, expected_encryption in [
+                (dict(file_id=a_id, destination_encryption=SSE_B2_AES), SSE_B2_AES),
+                (
+                    dict(
+                        file_id=a_id,
+                        destination_encryption=SSE_C_AES,
+                        file_info={'new': 'value'},
+                        content_type='text/plain'
+                    ), SSE_C_AES_NO_SECRET
+                ),
+                (
+                    dict(
+                        file_id=a_id,
+                        destination_encryption=SSE_C_AES,
+                        source_file_info={'old': 'value'},
+                        source_content_type='text/plain'
+                    ), SSE_C_AES_NO_SECRET
+                ),
+                (dict(file_id=b_id), SSE_NONE),
+                (dict(file_id=b_id, source_encryption=SSE_B2_AES), SSE_NONE),
+                (
+                    dict(
+                        file_id=b_id,
+                        source_encryption=SSE_B2_AES,
+                        destination_encryption=SSE_B2_AES
+                    ), SSE_B2_AES
+                ),
+                (
+                    dict(
+                        file_id=b_id,
+                        source_encryption=SSE_B2_AES,
+                        destination_encryption=SSE_C_AES,
+                        file_info={'new': 'value'},
+                        content_type='text/plain'
+                    ), SSE_C_AES_NO_SECRET
+                ),
+                (
+                    dict(
+                        file_id=b_id,
+                        source_encryption=SSE_B2_AES,
+                        destination_encryption=SSE_C_AES,
+                        source_file_info={'old': 'value'},
+                        source_content_type='text/plain'
+                    ), SSE_C_AES_NO_SECRET
+                ),
+                (
+                    dict(
+                        file_id=c_id,
+                        source_encryption=SSE_C_AES,
+                        file_info={'new': 'value'},
+                        content_type='text/plain'
+                    ), SSE_NONE
+                ),
+                (
+                    dict(
+                        file_id=c_id,
+                        source_encryption=SSE_C_AES,
+                        source_file_info={'old': 'value'},
+                        source_content_type='text/plain'
+                    ), SSE_NONE
+                ),
+                (
+                    dict(
+                        file_id=c_id, source_encryption=SSE_C_AES, destination_encryption=SSE_C_AES
+                    ), SSE_C_AES_NO_SECRET
+                ),
+                (
+                    dict(
+                        file_id=c_id,
+                        source_encryption=SSE_C_AES,
+                        destination_encryption=SSE_B2_AES,
+                        source_file_info={'old': 'value'},
+                        source_content_type='text/plain'
+                    ), SSE_B2_AES
+                ),
+                (
+                    dict(
+                        file_id=c_id,
+                        source_encryption=SSE_C_AES,
+                        destination_encryption=SSE_B2_AES,
+                        file_info={'new': 'value'},
+                        content_type='text/plain'
+                    ), SSE_B2_AES
+                ),
+                (
+                    dict(
+                        file_id=c_id,
+                        source_encryption=SSE_C_AES,
+                        destination_encryption=SSE_C_AES_2,
+                        source_file_info={'old': 'value'},
+                        source_content_type='text/plain'
+                    ), SSE_C_AES_2_NO_SECRET
+                ),
+                (
+                    dict(
+                        file_id=c_id,
+                        source_encryption=SSE_C_AES,
+                        destination_encryption=SSE_C_AES_2,
+                        file_info={'new': 'value'},
+                        content_type='text/plain'
+                    ), SSE_C_AES_2_NO_SECRET
+                ),
+            ]:
+                with self.subTest(kwargs=kwargs, length=length, data=data):
+                    file_info = self.bucket.copy(**kwargs, new_file_name='new_file', length=length)
+                    self.assertTrue(isinstance(file_info, FileVersionInfo))
+                    self.assertEqual(file_info.server_side_encryption, expected_encryption)
+
     def _make_file(self, bucket=None):
         data = b'hello world'
         actual_bucket = bucket or self.bucket
@@ -447,6 +719,65 @@ class TestUpload(TestCaseWithBucket):
         self.assertTrue(isinstance(file_info, FileVersionInfo))
         self._check_file_contents('file1', data)
 
+    def test_upload_bytes_file_retention(self):
+        data = b'hello world'
+        retention = FileRetentionSetting(RetentionMode.COMPLIANCE, 150)
+        file_info = self.bucket.upload_bytes(
+            data, 'file1', file_retention=retention, legal_hold=LegalHold.ON
+        )
+        self._check_file_contents('file1', data)
+        self.assertEqual(retention, file_info.file_retention)
+        self.assertEqual(LegalHold.ON, file_info.legal_hold)
+
+    def test_upload_bytes_sse_b2(self):
+        data = b'hello world'
+        file_info = self.bucket.upload_bytes(data, 'file1', encryption=SSE_B2_AES)
+        self.assertTrue(isinstance(file_info, FileVersionInfo))
+        self.assertEqual(file_info.server_side_encryption, SSE_B2_AES)
+
+    def test_upload_bytes_sse_c(self):
+        data = b'hello world'
+        file_info = self.bucket.upload_bytes(data, 'file1', encryption=SSE_C_AES)
+        self.assertTrue(isinstance(file_info, FileVersionInfo))
+        self.assertEqual(SSE_C_AES_NO_SECRET, file_info.server_side_encryption)
+
+    def test_upload_local_file_sse_b2(self):
+        with TempDir() as d:
+            path = os.path.join(d, 'file1')
+            data = b'hello world'
+            write_file(path, data)
+            file_info = self.bucket.upload_local_file(path, 'file1', encryption=SSE_B2_AES)
+            self.assertTrue(isinstance(file_info, FileVersionInfo))
+            self.assertEqual(file_info.server_side_encryption, SSE_B2_AES)
+            self._check_file_contents('file1', data)
+
+    def test_upload_local_file_sse_c(self):
+        with TempDir() as d:
+            path = os.path.join(d, 'file1')
+            data = b'hello world'
+            write_file(path, data)
+            file_info = self.bucket.upload_local_file(path, 'file1', encryption=SSE_C_AES)
+            self.assertTrue(isinstance(file_info, FileVersionInfo))
+            self.assertEqual(SSE_C_AES_NO_SECRET, file_info.server_side_encryption)
+            self._check_file_contents('file1', data)
+
+    def test_upload_local_file_retention(self):
+        with TempDir() as d:
+            path = os.path.join(d, 'file1')
+            data = b'hello world'
+            write_file(path, data)
+            retention = FileRetentionSetting(RetentionMode.COMPLIANCE, 150)
+            file_info = self.bucket.upload_local_file(
+                path,
+                'file1',
+                encryption=SSE_C_AES,
+                file_retention=retention,
+                legal_hold=LegalHold.ON
+            )
+            self._check_file_contents('file1', data)
+            self.assertEqual(retention, file_info.file_retention)
+            self.assertEqual(LegalHold.ON, file_info.legal_hold)
+
     def test_upload_bytes_progress(self):
         data = b'hello world'
         progress_listener = StubProgressListener()
@@ -603,35 +934,105 @@ class TestUpload(TestCaseWithBucket):
 
     def _check_file_contents(self, file_name, expected_contents):
         download = DownloadDestBytes()
-        self.bucket.download_file_by_name(file_name, download)
+        with FileSimulator.dont_check_encryption():
+            self.bucket.download_file_by_name(file_name, download)
         self.assertEqual(expected_contents, download.get_bytes_written())
 
-    def _make_data(self, approximate_length):
-        """
-        Generate a sequence of bytes to use in testing an upload.
-        Don't repeat a short pattern, so we're sure that the different
-        parts of a large file are actually different.
 
-        Returns bytes.
-        """
-        fragments = []
-        so_far = 0
-        while so_far < approximate_length:
-            fragment = ('%d:' % so_far).encode('utf-8')
-            so_far += len(fragment)
-            fragments.append(fragment)
-        return b''.join(fragments)
+class TestConcatenate(TestCaseWithBucket):
+    def _create_remote(self, sources, file_name, encryption=None):
+        return self.bucket.concatenate(sources, file_name=file_name, encryption=encryption)
+
+    def test_create_remote(self):
+        data = b'hello world'
+
+        f1_id = self.bucket.upload_bytes(data, 'f1').id_
+        f2_id = self.bucket.upload_bytes(data, 'f1').id_
+        with TempDir() as d:
+            path = os.path.join(d, 'file')
+            write_file(path, data)
+            created_file = self._create_remote(
+                [
+                    CopySource(f1_id, length=len(data), offset=0),
+                    UploadSourceLocalFile(path),
+                    CopySource(f2_id, length=len(data), offset=0),
+                ],
+                file_name='created_file'
+            )
+            self.assertIsInstance(created_file, FileVersionInfo)
+            actual = (
+                created_file.id_, created_file.file_name, created_file.size,
+                created_file.server_side_encryption
+            )
+            expected = ('9997', 'created_file', 33, SSE_NONE)
+            self.assertEqual(expected, actual)
+
+    def test_create_remote_encryption(self):
+        for data in [b'hello_world', self._make_data(self.simulator.MIN_PART_SIZE * 3)]:
+            f1_id = self.bucket.upload_bytes(data, 'f1', encryption=SSE_C_AES).id_
+            f2_id = self.bucket.upload_bytes(data, 'f1', encryption=SSE_C_AES_2).id_
+            with TempDir() as d:
+                path = os.path.join(d, 'file')
+                write_file(path, data)
+                created_file = self._create_remote(
+                    [
+                        CopySource(f1_id, length=len(data), offset=0, encryption=SSE_C_AES),
+                        UploadSourceLocalFile(path),
+                        CopySource(f2_id, length=len(data), offset=0, encryption=SSE_C_AES_2),
+                    ],
+                    file_name='created_file_%s' % (len(data),),
+                    encryption=SSE_C_AES
+                )
+            self.assertIsInstance(created_file, FileVersionInfo)
+            actual = (
+                created_file.id_, created_file.file_name, created_file.size,
+                created_file.server_side_encryption
+            )
+            expected = (
+                mock.ANY,
+                'created_file_%s' % (len(data),),
+                mock.ANY,  # FIXME: this should be equal to len(data) * 3,
+                # but there is a problem in the simulator/test code somewhere
+                SSE_C_AES_NO_SECRET
+            )
+            self.assertEqual(expected, actual)
+
+
+class TestCreateFile(TestConcatenate):
+    def _create_remote(self, sources, file_name, encryption=None):
+        return self.bucket.create_file(
+            [wi for wi in WriteIntent.wrap_sources_iterator(sources)],
+            file_name=file_name,
+            encryption=encryption
+        )
+
+
+class TestConcatenateStream(TestConcatenate):
+    def _create_remote(self, sources, file_name, encryption=None):
+        return self.bucket.concatenate_stream(sources, file_name=file_name, encryption=encryption)
+
+
+class TestCreateFileStream(TestConcatenate):
+    def _create_remote(self, sources, file_name, encryption=None):
+        return self.bucket.create_file_stream(
+            [wi for wi in WriteIntent.wrap_sources_iterator(sources)],
+            file_name=file_name,
+            encryption=encryption
+        )
 
 
 # Downloads
 
 
-class DownloadTests(object):
+class DownloadTests:
     DATA = 'abcdefghijklmnopqrs'
 
     def setUp(self):
         super(DownloadTests, self).setUp()
         self.file_info = self.bucket.upload_bytes(self.DATA.encode(), 'file1')
+        self.encrypted_file_info = self.bucket.upload_bytes(
+            self.DATA.encode(), 'enc_file1', encryption=SSE_C_AES
+        )
         self.download_dest = DownloadDestBytes()
         self.progress_listener = StubProgressListener()
 
@@ -744,6 +1145,18 @@ class DownloadTests(object):
             )
             self._check_local_file_contents(path, b'1234567defghij567890')
 
+    def test_download_by_id_no_progress_encryption(self):
+        self.bucket.download_file_by_id(
+            self.encrypted_file_info.id_, self.download_dest, encryption=SSE_C_AES
+        )
+        self._verify(self.DATA, check_progress_listener=False)
+
+    def test_download_by_id_no_progress_wrong_encryption(self):
+        with self.assertRaises(SSECKeyError):
+            self.bucket.download_file_by_id(
+                self.encrypted_file_info.id_, self.download_dest, encryption=SSE_C_AES_2
+            )
+
     def _check_local_file_contents(self, path, expected_contents):
         with open(path, 'rb') as f:
             contents = f.read()
@@ -753,7 +1166,7 @@ class DownloadTests(object):
 # download empty file
 
 
-class EmptyFileDownloadScenarioMixin(object):
+class EmptyFileDownloadScenarioMixin:
     """ use with DownloadTests, but not for TestDownloadParallel as it does not like empty files """
 
     def test_download_by_name_empty_file(self):
@@ -772,15 +1185,15 @@ class TestDownloadDefault(DownloadTests, EmptyFileDownloadScenarioMixin, TestCas
 class TestDownloadSimple(DownloadTests, EmptyFileDownloadScenarioMixin, TestCaseWithBucket):
     def setUp(self):
         super(TestDownloadSimple, self).setUp()
-        self.bucket.api.services.download_manager.strategies = [
-            SimpleDownloader(force_chunk_size=20,)
-        ]
+        download_manager = self.bucket.api.services.download_manager
+        download_manager.strategies = [SimpleDownloader(force_chunk_size=20)]
 
 
 class TestDownloadParallel(DownloadTests, TestCaseWithBucket):
     def setUp(self):
         super(TestDownloadParallel, self).setUp()
-        self.bucket.api.services.download_manager.strategies = [
+        download_manager = self.bucket.api.services.download_manager
+        download_manager.strategies = [
             ParallelDownloader(
                 force_chunk_size=2,
                 max_streams=999,
@@ -821,15 +1234,15 @@ class TestCaseWithTruncatedDownloadBucket(TestCaseWithBucket):
 class TestTruncatedDownloadSimple(DownloadTests, TestCaseWithTruncatedDownloadBucket):
     def setUp(self):
         super(TestTruncatedDownloadSimple, self).setUp()
-        self.bucket.api.services.download_manager.strategies = [
-            SimpleDownloader(force_chunk_size=20,)
-        ]
+        download_manager = self.bucket.api.services.download_manager
+        download_manager.strategies = [SimpleDownloader(force_chunk_size=20)]
 
 
 class TestTruncatedDownloadParallel(DownloadTests, TestCaseWithTruncatedDownloadBucket):
     def setUp(self):
         super(TestTruncatedDownloadParallel, self).setUp()
-        self.bucket.api.services.download_manager.strategies = [
+        download_manager = self.bucket.api.services.download_manager
+        download_manager.strategies = [
             ParallelDownloader(
                 force_chunk_size=3,
                 max_streams=2,
