@@ -8,4 +8,4 @@
 #
 ######################################################################
 
-from b2sdk.v1.exception import *
+from b2sdk.v1.exception import *  # noqa
